/*
This is a hacked version of:
Capacitive Synth project by Other Machine Co. http://othermachine.co
Capacitive touch code from http://playground.arduino.cc/Code/CapacitiveSensor
Tone code from http://arduino.cc/en/reference/tone

Hacked for the ArduTouch music synthesizer PCB (RevA mono).

This plays a different note depending on which key you touch on the ArduTouch PCB.
Hitting the upper push-button switch raised the octave.
Hitting the lower push-button switch lowers the octave.


Hardware:
    Speaker with LM386 amplifier on Pin 9

    Capacitive keyboard:
        C   Pin 13
        C#  Pin 12
        D   Pin 11
        D#  Pin 10
        E   Pin 8
        F   Pin 7
        F#  Pin 6
        G   Pin 5
        G#  Pin 4
        A   Pin 3
        A#  Pin 2
        B   Pin 1
        C   Pin 19 (Analog Pin 5)

    LED on Pin 0

    Push-button switch (upper switch: S3) on Analog Pin 3
    Push-button switch (lower switch: S2) on Analog Pin 2

*/

#include "pitches.h"
#include <avr/pgmspace.h>

const int NOTELENGTH  = 20;    // length of time to play a note for the tone() function
const int OUTPUTPIN   = 9;     // the tone() function outputs its squarewave to this pin
const int CAPACITANCE = 2;     // sensitivity for readCapacitivePin(), the capacitance touch function (2 is the only value that works well)
const int LEDPIN = 0;          // an LED is connected to this pin
const int SWITCHPIN_S3 = 17;   // the upper push-button switch is connected to this pin -- analog Pin 3 (which can be used as digital Pin 17)
const int SWITCHPIN_S2 = 16;   // the lower push-button switch is connected to this pin -- analog Pin 2 (which can be used as digital Pin 16)

int S2_pressed = 0;        // keeps track of whether switch S2 was pressed (to go down an octave) -- S2 is 0 if it is not being pressed
int S3_pressed = 0;        // keeps track of whether switch S3 was pressed (to go up an octave) -- S3 is 0 if it is not being pressed
int S2_still_pressed = 0;  // keeps track of whether switch S2 was pressed and is still pressed (without being released) -- this will become 1 when S2 is pressed, and 0 when S2 is released
int S3_still_pressed = 0;  // keeps track of whether switch S3 was pressed and is still pressed (without being released) -- this will become 1 when S3 is pressed, and 0 when S3 is released
int octave = 4;            // keep track of which octave to play -- start at octave 4 (the middle octave)


// This array contains the notes for 7 octaves of notes
//   (We want this large array to be in program memory, which is why we use PROGMEM here.)
const PROGMEM uint16_t noteArray[] = {
  NOTE_C1, NOTE_CS1, NOTE_D1, NOTE_DS1, NOTE_E1, NOTE_F1, NOTE_FS1, NOTE_G1, NOTE_GS1, NOTE_A1, NOTE_AS1, NOTE_B1, NOTE_C2,
  NOTE_C2, NOTE_CS2, NOTE_D2, NOTE_DS2, NOTE_E2, NOTE_F2, NOTE_FS2, NOTE_G2, NOTE_GS2, NOTE_A2, NOTE_AS2, NOTE_B2, NOTE_C3,
  NOTE_C3, NOTE_CS3, NOTE_D3, NOTE_DS3, NOTE_E3, NOTE_F3, NOTE_FS3, NOTE_G3, NOTE_GS3, NOTE_A3, NOTE_AS3, NOTE_B3, NOTE_C4,
  NOTE_C4, NOTE_CS4, NOTE_D4, NOTE_DS4, NOTE_E4, NOTE_F4, NOTE_FS4, NOTE_G4, NOTE_GS4, NOTE_A4, NOTE_AS4, NOTE_B4, NOTE_C5,
  NOTE_C5, NOTE_CS5, NOTE_D5, NOTE_DS5, NOTE_E5, NOTE_F5, NOTE_FS5, NOTE_G5, NOTE_GS5, NOTE_A5, NOTE_AS5, NOTE_B5, NOTE_C6,
  NOTE_C6, NOTE_CS6, NOTE_D6, NOTE_DS6, NOTE_E6, NOTE_F6, NOTE_FS6, NOTE_G6, NOTE_GS6, NOTE_A6, NOTE_AS6, NOTE_B6, NOTE_C7,
  NOTE_C7, NOTE_CS7, NOTE_D7, NOTE_DS7, NOTE_E7, NOTE_F7, NOTE_FS7, NOTE_G7, NOTE_GS7, NOTE_A7, NOTE_AS7, NOTE_B7, NOTE_C8,
};


//-----------------------------------------------------------------------------------

// the code in setup() is executed once after power up (or reset button is pressed)
void setup(void)
{
  // enable the LED pin as an output so we can turn the LED connected to it on and off
  pinMode(LEDPIN, OUTPUT); 
  // enable the two switch pins as inputs, with pullup resistors active (so that no button press yields a High, and a button press yields a Low)
  pinMode(SWITCHPIN_S2, INPUT_PULLUP);
  pinMode(SWITCHPIN_S3, INPUT_PULLUP);

  //beep speaker and pulse LED to show that the device is working
  digitalWrite(LEDPIN, HIGH);     // turn the LED on
  tone(OUTPUTPIN, NOTE_A4, 500);  // beep the speaker
  delay(100);                     // wait a little while before
  digitalWrite(LEDPIN, LOW);      //    turning the LED off

  Serial.end();
}


//-----------------------------------------------------------------------------------


// the code in loop() is done repeatedly until power goes off (or reset button is pressed)
void loop(void)
{
  // wait for a touch of a key, and play the note for that key
  // if a key was touched, then turn on the LED, otherwise turn it off
  if ( capKeyboardOctave() ) {
      digitalWrite(LEDPIN, HIGH);     // turn the LED on
  }
  else {
      digitalWrite(LEDPIN, LOW);      // turn the LED off
  }

  // read the two push buttons, and mark them a pressed or not pressed
  if ( digitalRead(SWITCHPIN_S2) )
    S2_pressed = 0;  // S2 is not being pressed
  else
    S2_pressed = 1;  // S2 is being pressed
  if ( digitalRead(SWITCHPIN_S3) )
    S3_pressed = 0;  // S3 is not being pressed
  else
    S3_pressed = 1;  // S3 is being pressed
  
  // if the bottom button (S2) is pressed and isn't still pressed from the last time through the loop, then go down an octave
  if ( S2_pressed && !S2_still_pressed ) {
    octave--;
    // don't allow going below octave 1;
    if ( octave < 1 ) octave = 1;
    S2_still_pressed = 1;  // mark S2_still_pressed as true (for the next time through the loop)
    // pulse the LED to show that the button has been pressed
    digitalWrite(LEDPIN, HIGH);     // turn the LED on
    delay(50);
    digitalWrite(LEDPIN, LOW);      // turn the LED off
  }
  // if S2 has been released then mark S2_still_pressed as false
  if ( !S2_pressed && S2_still_pressed ) {
    S2_still_pressed = 0;
  }
  
  // if the top button (S3) is pressed and isn't still pressed from the last time through the loop, then go up an octave
  if ( S3_pressed && !S3_still_pressed ) {
    octave++;
    // don't allow going above octave 7;
    if ( octave > 7 ) octave = 7;
    S3_still_pressed = 1;  // mark S3_still_pressed as true (for the next time through the loop)
    // pulse the LED to show that the button has been pressed
    digitalWrite(LEDPIN, HIGH);     // turn the LED on
    delay(50);
    digitalWrite(LEDPIN, LOW);      // turn the LED off
  }
  // if S3 has been released then mark S3_still_pressed as false
  if ( !S3_pressed && S3_still_pressed ) {
    S3_still_pressed = 0;
  }
}


//-----------------------------------------------------------------------------------



/////////////////////////////////////////////////////////////////////////////////////
//
// readCapacitivePin
//
//  Input: Arduino pin number
//  Output: A number, from 0 to 17 expressing how much capacitance is on the pin
//          I've found that using 0 or 1 is too sensitive, 
//            and using 3 is just a bit too little sensitivity, but that 2 is just right (Mitch)
//
//  To see if someone is touching a key (which is connected to an Arduino pin):
//     make the pin Low as an output pin (this discharges any capacitance on the pin),
//     then make the pin an input pin (so the capacitance on the pin starts charging),
//     and see how long the pin takes to charge up to a High level.
//     
//     Touching a key (and thus the pin) adds capacitance to the pin, 
//     which makes it take longer to charge up to a High level.
//     The output of this function is a quantized version of how long the pin took to charge to a High level.
//     If no one is touching a key, the output will be below 3.
//     If someone is touching a key, the output will probably be 3.
//     
/////////////////////////////////////////////////////////////////////////////////////

#include "pins_arduino.h" // Arduino pre-1.0 needs this

uint8_t readCapacitivePin(int pinToMeasure) {

  // Variables used to translate from Arduino to AVR pin naming
  volatile uint8_t* port;
  volatile uint8_t* ddr;
  volatile uint8_t* pin;
  // Here we translate the input pin number from Arduino pin number
  //     to the AVR PORT, PIN, DDR,
  //     and which bit of those registers we care about.
  byte bitmask;
  port = portOutputRegister( digitalPinToPort(pinToMeasure) );
  ddr = portModeRegister( digitalPinToPort(pinToMeasure) );
  bitmask = digitalPinToBitMask(pinToMeasure);
  pin = portInputRegister( digitalPinToPort(pinToMeasure) );
  // Discharge the pin first by setting it low and output
  *port &= ~(bitmask);
  *ddr  |= bitmask;
  delay(1);
  
  // Prevent the timer IRQ from disturbing our timing measurement
  noInterrupts();
  
  // Make the pin an input with the internal pull-up on
  *ddr &= ~(bitmask);
  *port |= bitmask;

  // Now see how long the pin takes to charge to a High level. 
  // The following manual unrolling of the loop 
  //   decreases the number of hardware cycles between each read of the pin,
  //   thus increasing sensitivity.
  uint8_t cycles = 17;
  if (*pin & bitmask) {
    cycles =  0;
  }
  else if (*pin & bitmask) {
    cycles =  1;
  }
  else if (*pin & bitmask) {
    cycles =  2;
  }
  else if (*pin & bitmask) {
    cycles =  3;
  }
  else if (*pin & bitmask) {
    cycles =  4;
  }
  else if (*pin & bitmask) {
    cycles =  5;
  }
  else if (*pin & bitmask) {
    cycles =  6;
  }
  else if (*pin & bitmask) {
    cycles =  7;
  }
  else if (*pin & bitmask) {
    cycles =  8;
  }
  else if (*pin & bitmask) {
    cycles =  9;
  }
  else if (*pin & bitmask) {
    cycles = 10;
  }
  else if (*pin & bitmask) {
    cycles = 11;
  }
  else if (*pin & bitmask) {
    cycles = 12;
  }
  else if (*pin & bitmask) {
    cycles = 13;
  }
  else if (*pin & bitmask) {
    cycles = 14;
  }
  else if (*pin & bitmask) {
    cycles = 15;
  }
  else if (*pin & bitmask) {
    cycles = 16;
  }

  // End of timing-critical section
  interrupts();

  // Discharge the pin again by setting it low and output
  //  It's important to leave the pins low if you want to
  //  be able to touch more than 1 sensor at a time - if
  //  the sensor is left pulled high, when you touch
  //  two sensors, your body will transfer the charge between
  //  sensors.
  *port &= ~(bitmask);
  *ddr  |= bitmask;

  return cycles;
}



/////////////////////////////////////////////////////////////////////////////////////
//
// capKeyboardOctave
//
// This function plays a square wave of the correct pitch 
//     depending on which key of the keyboard is touched:
//         C, C#, D, D#, E, F, F#, G, G#, A, A#, B, C
//
// noteArray[] contains the correct values for the above notes for all octaves.
//
/////////////////////////////////////////////////////////////////////////////////////

int capKeyboardOctave(void)
{
int keytouch = 0;
  // play a C in the chosen octave if the C key is being touched
  if (readCapacitivePin(13) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 0 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a C# in the chosen octave if the C key is being touched
  if (readCapacitivePin(12) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 1 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a D in the chosen octave if the C key is being touched
  if (readCapacitivePin(11) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 2 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a D# in the chosen octave if the C key is being touched
  if (readCapacitivePin(10) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 3 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a E in the chosen octave if the C key is being touched
  if (readCapacitivePin(8) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 4 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a F in the chosen octave if the C key is being touched
  if (readCapacitivePin(7) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 5 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a F# in the chosen octave if the C key is being touched
  if (readCapacitivePin(6) > CAPACITANCE)   {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 6 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a G in the chosen octave if the C key is being touched
  if (readCapacitivePin(5) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 7 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a G# in the chosen octave if the C key is being touched
  if (readCapacitivePin(4) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 8 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a A in the chosen octave if the C key is being touched
  if (readCapacitivePin(3) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 9 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a A# in the chosen octave if the C key is being touched
  if (readCapacitivePin(2) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 10 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a B in the chosen octave if the C key is being touched
  if (readCapacitivePin(1) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 11 ), NOTELENGTH);
    keytouch = 1;
  }
  // play a C in the chosen octave if the C key is being touched
  if (readCapacitivePin(19) > CAPACITANCE)  {
    tone(OUTPUTPIN, pgm_read_word_near( noteArray + ((octave-1)*13) + 12 ), NOTELENGTH);
    keytouch = 1;
  }
  return keytouch;
}

